(function () {
  'use strict';

  function copyStatics(to, from) {
    to.currentInput = from.currentInput;
    to.version = from.version;
    to.defaultProps = from.defaultProps;
    to.setDefaultProps = from.setDefaultProps;
    to.hideAll = from.hideAll;
    return to;
  }

  function enhance(tippyBase, propHOFs) {
    var tippyEnhanced = propHOFs.reduce(function (nextTippy, addProp) {
      return addProp(nextTippy);
    }, tippyBase);
    return copyStatics(tippyEnhanced, tippyBase);
  }

  function _extends() {
    _extends = Object.assign || function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];

        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }

      return target;
    };

    return _extends.apply(this, arguments);
  }

  var isBrowser = typeof window !== 'undefined' && typeof document !== 'undefined';
  var ua = isBrowser ? navigator.userAgent : '';
  var isIE = /MSIE |Trident\//.test(ua);
  var isUCBrowser = /UCBrowser\//.test(ua);
  var isIOS = isBrowser && /iPhone|iPad|iPod/.test(navigator.platform);

  /**
   * Safe .hasOwnProperty check, for prototype-less objects
   */

  function hasOwnProperty(obj, key) {
    return {}.hasOwnProperty.call(obj, key);
  }
  /**
   * Returns offsets for virtual reference when using a cursor point
   */

  function getVirtualOffsets(_ref, isVerticalPlacement) {
    var popper = _ref.popper,
        currentPlacement = _ref.state.currentPlacement;
    var isShift = !!currentPlacement.split('-')[1];
    var size = isVerticalPlacement ? popper.offsetWidth : popper.offsetHeight;
    var halfSize = size / 2;
    return {
      size: size,
      x: isVerticalPlacement ? isShift ? size : halfSize : 0,
      y: isVerticalPlacement ? 0 : isShift ? size : halfSize
    };
  }
  /**
   * Preserves the original function invocation when another function replaces it
   */

  function preserveInvocation(originalFn, currentFn, args) {
    if (originalFn && originalFn !== currentFn) {
      originalFn.apply(null, args);
    }
  }
  /**
   * Deletes properties from an object (pure)
   */

  function removeProperties(partialProps, keys) {
    var clone = _extends({}, partialProps);

    keys.forEach(function (key) {
      delete clone[key];
    });
    return clone;
  }
  /**
   * Ponyfill for Array.from - converts iterable values to an array
   */

  function arrayFrom(value) {
    return [].slice.call(value);
  }
  /**
   * Works like Element.prototype.closest, but uses a callback instead
   */

  function closestCallback(element, callback) {
    while (element) {
      if (callback(element)) {
        return element;
      }

      element = element.parentElement;
    }

    return null;
  }
  /**
   * Determines if an array or string includes a value
   */

  function includes(a, b) {
    return a.indexOf(b) > -1;
  }

  /**
   * Returns the popper's placement, ignoring shifting (top-start, etc)
   */

  function getBasePlacement(placement) {
    return placement.split('-')[0];
  }

  var currentInput = {
    isTouch: false
  };

  function withFollowCursor(tippy) {
    return function (targets, optionalProps) {
      return tippy(targets, _extends({}, optionalProps, {
        onCreate: function onCreate(instance) {
          preserveInvocation(optionalProps && optionalProps.onCreate, instance.props.onCreate, [instance]);

          {
            instance.__extraProps__.followCursor = true;
          }

          var undo = function undo() {};

          var originalSetProps = instance.setProps;

          instance.setProps = function (partialProps) {
            if (hasOwnProperty(partialProps, 'followCursor')) {
              undo();

              if (partialProps.followCursor) {
                undo = applyFollowCursor(instance);
              }
            }

            originalSetProps(partialProps);
          };

          if (instance.props.followCursor) {
            undo = applyFollowCursor(instance);
          }
        }
      }));
    };
  }

  function applyFollowCursor(instance) {
    var reference = instance.reference;
    var lastMouseMoveEvent;
    var isPopperInstanceCreated = false;
    var isDestroyed = false;
    var wasTriggeredByFocus = false;

    function addListener() {
      document.addEventListener('mousemove', onMouseMove);
    }

    function removeListener() {
      document.removeEventListener('mousemove', onMouseMove);
    }

    function onMouseMove(event) {
      if (wasTriggeredByFocus || isDestroyed || !event) {
        return;
      }

      var _lastMouseMoveEvent = lastMouseMoveEvent = event,
          clientX = _lastMouseMoveEvent.clientX,
          clientY = _lastMouseMoveEvent.clientY;

      if (!instance.popperInstance || !isPopperInstanceCreated) {
        return;
      } // If the instance is interactive, avoid updating the position unless it's
      // over the reference element


      var isCursorOverReference = closestCallback(event.target, function (el) {
        return el === reference;
      });
      var rect = reference.getBoundingClientRect();
      var followCursor = instance.props.followCursor;
      var isHorizontal = followCursor === 'horizontal';
      var isVertical = followCursor === 'vertical';
      var isVerticalPlacement = includes(['top', 'bottom'], getBasePlacement(instance.state.currentPlacement)); // The virtual reference needs some size to prevent itself from overflowing

      var _getVirtualOffsets = getVirtualOffsets(instance, isVerticalPlacement),
          size = _getVirtualOffsets.size,
          x = _getVirtualOffsets.x,
          y = _getVirtualOffsets.y;

      if (isCursorOverReference || !instance.props.interactive) {
        instance.popperInstance.reference = {
          // These `client` values don't get used by Popper.js if they are 0
          clientWidth: 0,
          clientHeight: 0,
          getBoundingClientRect: function getBoundingClientRect() {
            return {
              width: isVerticalPlacement ? size : 0,
              height: isVerticalPlacement ? 0 : size,
              top: (isHorizontal ? rect.top : clientY) - y,
              bottom: (isHorizontal ? rect.bottom : clientY) + y,
              left: (isVertical ? rect.left : clientX) - x,
              right: (isVertical ? rect.right : clientX) + x
            };
          }
        };
        instance.popperInstance.update();
      } // IIFE context vs. ESM/CJS context


      var ci = // @ts-ignore
      // eslint-disable-next-line
      typeof tippy !== 'undefined' ? tippy.currentInput : currentInput;

      if (ci.isTouch) {
        removeListener();
      }
    }

    var _instance$props = instance.props,
        popperOptions = _instance$props.popperOptions,
        _onMount = _instance$props.onMount,
        _onTrigger = _instance$props.onTrigger,
        _onUntrigger = _instance$props.onUntrigger,
        _onHidden = _instance$props.onHidden;
    instance.setProps({
      popperOptions: {
        onCreate: function onCreate(data) {
          preserveInvocation(popperOptions && popperOptions.onCreate, instance.props.popperOptions.onCreate, [data]);
          isPopperInstanceCreated = true;
        }
      },
      onMount: function onMount(instance) {
        preserveInvocation(_onMount, instance.props.onMount, [instance]); // Popper's scroll listeners make sense for `true`, where the cursor
        // follows both axes. TODO: somehow keep scroll listeners for vertical
        // scrolling for "vertical", and horizontal scrolling for "horizontal".

        if (!wasTriggeredByFocus && instance.props.followCursor !== true) {
          instance.popperInstance.disableEventListeners();
        }

        onMouseMove(lastMouseMoveEvent);
      },
      onTrigger: function onTrigger(instance, event) {
        preserveInvocation(_onTrigger, instance.props.onTrigger, [instance, event]);
        wasTriggeredByFocus = event.type === 'focus';

        if (event instanceof MouseEvent) {
          lastMouseMoveEvent = event;
        }

        if (wasTriggeredByFocus && instance.popperInstance) {
          instance.popperInstance.reference = instance.reference;
        }

        if (!wasTriggeredByFocus && instance.props.followCursor !== 'initial') {
          addListener();
        }
      },
      onUntrigger: function onUntrigger(instance, event) {
        preserveInvocation(_onUntrigger, instance.props.onUntrigger, [instance, event]); // The listener gets added in `onTrigger()`, but due to potential delay(s)
        // the instance made be untriggered before it shows. `onHidden()` will
        // therefore never be invoked.

        if (!instance.state.isVisible) {
          removeListener();
        }
      },
      onHidden: function onHidden() {
        preserveInvocation(_onHidden, instance.props.onHidden, [instance]); // If scheduled to show before unmounting (e.g. delay: [500, 0]), the
        // listener should not be removed

        if (!instance.state.isScheduledToShow) {
          removeListener();
        }
      }
    });
    var originalSetProps = instance.setProps;

    instance.setProps = function (partialProps) {
      _onTrigger = partialProps.onTrigger || _onTrigger;
      _onUntrigger = partialProps.onUntrigger || _onUntrigger;
      _onMount = partialProps.onMount || _onMount;
      _onHidden = partialProps.onHidden || _onHidden;
      popperOptions = partialProps.popperOptions || popperOptions;
      originalSetProps(removeProperties(partialProps, ['onTrigger', 'onUntrigger', 'onMount', 'onHidden', 'popperOptions']));
      onMouseMove(lastMouseMoveEvent);
    };

    return function () {
      // Undo
      removeListener();

      if (instance.popperInstance) {
        instance.popperInstance.reference = instance.reference;
      }

      instance.setProps = originalSetProps;
      originalSetProps({
        popperOptions: popperOptions,
        onTrigger: _onTrigger,
        onUntrigger: _onUntrigger,
        onMount: _onMount,
        onHidden: _onHidden
      });
      isDestroyed = true;
    };
  }

  /**
   * Helpful wrapper around `console.warn()`.
   * TODO: Should we use a cache so it only warns a single time and not spam the
   * console? (Need to consider hot reloading and invalidation though). Chrome
   * already batches warnings as well.
   */

  function warnWhen(condition, message) {
    if (condition) {
      /* eslint-disable-next-line no-console */
      console.warn("[tippy.js WARNING] " + message);
    }
  }

  function withInlinePositioning(tippy) {
    return function (targets, optionalProps) {
      var props = _extends({
        inlinePositioning: false
      }, optionalProps);

      return tippy(targets, _extends({}, props, {
        onCreate: function onCreate(instance) {
          preserveInvocation(optionalProps && optionalProps.onCreate, instance.props.onCreate, [instance]);

          {
            instance.__extraProps__.inlinePositioning = true;
          }

          if (props.inlinePositioning) {
            var virtualReference = document.createElement('div');
            var _onTrigger = instance.props.onTrigger;
            instance.setProps({
              onTrigger: function onTrigger(instance, event) {
                preserveInvocation(_onTrigger, instance.props.onTrigger, [instance, event]);
                instance.popperInstance.reference = virtualReference;
              }
            });

            if (props.inlinePositioning === 'cursor') {
              applyCursorStrategy(instance);
            } else {
              virtualReference.getBoundingClientRect = function () {
                return getBestRect(instance);
              };
            }

            var originalSetProps = instance.setProps;

            instance.setProps = function (partialProps) {
              // Making this prop fully dynamic is difficult and buggy, and it's
              // very unlikely the user will need to dynamically update it anyway.
              // Just warn.
              {
                warnWhen(hasOwnProperty(partialProps, 'inlinePositioning'), 'Cannot change `inlinePositioning` prop. Destroy this ' + 'instance and create a new instance instead.');
              }

              _onTrigger = partialProps.onTrigger || _onTrigger;
              originalSetProps(removeProperties(partialProps, ['onTrigger']));
            };
          }
        }
      }));
    };
  }
  function getBestRect(instance) {
    var reference = instance.reference;
    var rects = reference.getClientRects();
    var basePlacement = getBasePlacement(instance.state.currentPlacement);
    var top;
    var right;
    var bottom;
    var left; // Not an inline element that spans 2 or more rows

    if (rects.length < 2) {
      return reference.getBoundingClientRect();
    }

    var firstRect = rects[0];
    var lastRect = rects[rects.length - 1];

    switch (basePlacement) {
      case 'top':
        {
          top = firstRect.top;
          bottom = firstRect.bottom;
          left = firstRect.left;
          right = firstRect.right;
          break;
        }

      case 'bottom':
        {
          top = lastRect.top;
          bottom = lastRect.bottom;
          left = lastRect.left;
          right = lastRect.right;
          break;
        }

      case 'left':
      case 'right':
        {
          var rectsArr = arrayFrom(rects); // The values are rounded because the rects are displayed via whole pixels
          // e.g. 140.1 and 139.9 are aligned the same

          var lefts = rectsArr.map(function (rects) {
            return Math.round(rects.left);
          });
          var rights = rectsArr.map(function (rects) {
            return Math.round(rects.right);
          });
          var minLeft = Math.min.apply(Math, lefts);
          var maxRight = Math.max.apply(Math, rights);
          var measureRects = rectsArr.filter(function (rect) {
            return basePlacement === 'left' ? Math.round(rect.left) === minLeft : Math.round(rect.right) === maxRight;
          });
          top = measureRects[0].top;
          bottom = measureRects[measureRects.length - 1].bottom;
          left = minLeft;
          right = maxRight;
          break;
        }

      default:
        {
          return reference.getBoundingClientRect();
        }
    }

    return {
      width: right - left,
      height: bottom - top,
      top: top,
      right: right,
      bottom: bottom,
      left: left
    };
  }
  function applyCursorStrategy(instance) {
    var reference = instance.reference;
    var _onTrigger2 = instance.props.onTrigger;
    instance.setProps({
      onTrigger: function onTrigger(instance, event) {
        preserveInvocation(_onTrigger2, instance.props.onTrigger, [instance, event]);
        var rects = arrayFrom(reference.getClientRects());
        var basePlacement = getBasePlacement(instance.state.currentPlacement);

        if (event instanceof MouseEvent) {
          // We need to choose which rect to use. Check which rect
          // the cursor landed on.
          var index = -1;
          rects.forEach(function (rect, i) {
            var isVerticalPlacement = includes(['top', 'bottom'], basePlacement);
            var isWithinHorizontalBounds = event.clientX >= Math.floor(rect.left) && event.clientX <= Math.ceil(rect.right);
            var isWithinAllBounds = isWithinHorizontalBounds && event.clientY >= Math.floor(rect.top) && event.clientY <= Math.ceil(rect.bottom);

            if (isVerticalPlacement) {
              if (isWithinHorizontalBounds && (basePlacement === 'top' && index === -1 || basePlacement === 'bottom')) {
                index = i;
              }
            } else if (isWithinAllBounds && (basePlacement === 'left' && index === -1 || basePlacement === 'right')) {
              index = i;
            }
          });

          instance.popperInstance.reference.getBoundingClientRect = function () {
            var rect = reference.getClientRects()[index];
            var isVerticalPlacement = includes(['top', 'bottom'], getBasePlacement(instance.state.currentPlacement));

            var _getVirtualOffsets = getVirtualOffsets(instance, isVerticalPlacement),
                size = _getVirtualOffsets.size,
                x = _getVirtualOffsets.x,
                y = _getVirtualOffsets.y;

            var baseRect = {
              width: isVerticalPlacement ? size : 0,
              height: isVerticalPlacement ? 0 : size,
              top: rect.top,
              right: rect.right,
              bottom: rect.bottom,
              left: rect.left
            };
            return isVerticalPlacement ? _extends({}, baseRect, {
              left: event.clientX - x,
              right: event.clientX + x
            }) : _extends({}, baseRect, {
              top: event.clientY - y,
              bottom: event.clientY + y
            });
          };
        } else {
          // Fallback to `getBestRect` since "cursor" coords don't apply to
          // non-MouseEvents
          instance.popperInstance.reference.getBoundingClientRect = function () {
            return getBestRect(instance);
          };
        }
      }
    });
    var originalSetProps = instance.setProps;

    instance.setProps = function (partialProps) {
      _onTrigger2 = partialProps.onTrigger || _onTrigger2;
      originalSetProps(removeProperties(partialProps, ['onTrigger']));
    };
  }

  /* eslint-disable no-undef */

  if (typeof tippy === 'function') {
    window.tippy = enhance(tippy, [withFollowCursor, withInlinePositioning]);
  } else {
    throw new Error('[tippy.js ERROR] `tippy` is not a global function. Make sure you have ' + 'included the tippy script before tippy-extra-props.');
  }

}());
//# sourceMappingURL=tippy-extra-props.js.map
